@@ -96,11 +96,16 @@ def get_markup_validity(file_path: str) -> list:
         headers = {
             "content-type": "text/html; charset=utf-8",
             "Accept-Charset": "UTF-8",
+            "User-Agent": "python-requests/2.32.5",
         }
 
         r = requests.post(w3cURL, data=payload, headers=headers)
-        errors = r.json()
-        errors = errors.get("messages")
+        try:
+            errors = r.json()
+            errors = errors.get("messages")
+        except Exception:
+            # We need to use the web browser
+            errors = get_validation_by_browser(file_path)
 
         # raise the alarm if the response code is not 200
         if r.status_code != 200:
@@ -220,6 +225,45 @@ def is_css_valid(validator_results):
     return bool(soup.find(id="congrats"))
 
 
+def get_validation_by_browser(file_path: str) -> list:
+    """Validates HTML using the browser.
+
+    This function will get the HTML code by file and use the mechanical
+    soup browser to get validator results as a ResultSet.
+
+    Args:
+        css_code: CSS code in the form of a string.
+
+    Returns:
+        results: A ResultSet of Tag objects.
+    """
+    # Get HTML code from file path
+    html_code = clerk.file_to_string(file_path)
+    try:
+        response = browser.open("https://validator.w3.org/")
+        if not response.ok:
+            response = browser.open("https://validator.w3.org/nu/#textarea")
+            browser.select_form("form")
+            browser["doc"] = html_code
+            browser.submit_selected()
+            results = browser.get_current_page().select("div#results")
+        else:
+            browser.select_form("#validate-by-input form")
+            browser["fragment"] = html_code
+            browser.submit_selected()
+            results = browser.get_current_page().select("div#results")
+    except Exception:
+        # Convert the file "no_css_connection.html" into a soup tag object
+        no_connection_code = (
+            "<h1>Sorry, but we could not make a connection</h1>"
+        )
+        no_connection_code += "<h2>Please try later</h2>"
+        soup = BeautifulSoup(no_connection_code, "lxml")
+        # Convert string to result set
+        results = soup.contents
+    return results
+
+
 def validate_css(css_code: str) -> bs4.ResultSet:
     """Validates CSS and returns the results from the css-validator.
 
@@ -262,6 +306,7 @@ def get_project_validation(project_dir: str, type="html") -> list:
     will return a list of per-files errors
     """
     report = []
+    passing_files = []
     all_files = clerk.get_all_project_files(project_dir)
     for file in all_files:
         errors = []
@@ -274,24 +319,37 @@ def get_project_validation(project_dir: str, type="html") -> list:
                     f"fail: {filename} has {len(errors)} validation errors."
                 )
             else:
-                report.append(f"pass: {filename} passes HTML validation!")
+                passing_files.append(filename)
         else:
-            if file_type == "html":
+            if file_type == "html" and type == "css":
                 style_tag = html.get_elements("style", file)
                 if style_tag:
                     code = html.get_element_content(style_tag)
                     result = validate_css(code)
-                    errors += get_css_errors_list(result)
+                    errors_list = get_css_errors_list(result)
+                    if errors_list:
+                        errors += errors_list
+                    else:
+                        passing_files.append(filename)
             if file_type == "css":
                 code = clerk.file_to_string(file)
                 result = validate_css(code)
-                errors += get_css_errors_list(result)
+                errors_list = get_css_errors_list(result)
+                if errors_list:
+                    errors += errors_list
+                else:
+                    passing_files.append(filename)
             if errors:
                 report.append(
                     f"fail: {filename} has {len(errors)} css errors."
                 )
     if not report:
-        report.append("fail: no files present to validate")
+        if passing_files:
+            for passing_file in passing_files:
+                msg = f"pass: {passing_file} passes {type.upper()} validation"
+                report.append(msg)
+        if not passing_files:
+            report.append("fail: no files present to validate")
     # TODO - make sure this covers all scenarios
     return report
 
